@@ -17,6 +17,7 @@ __date__ ="$Sep 23, 2011M$"
 import math
 import itertools
 import logging
+import random
 from collections import OrderedDict
 
 import numpy as np
@@ -26,6 +27,8 @@ from pymatgen.core.structure import Structure
 from pymatgen.core.operations import SymmOp
 from pymatgen.core.structure_modifier import StructureEditor
 
+logger = logging.getLogger(__name__)
+
 class StructureFitter(object):
     """
     Class to perform fitting of two structures
@@ -36,8 +39,12 @@ class StructureFitter(object):
         structure_a - First structure
         structure_b - Second structure
     """
+    FAST_FIT = 25000
+    NORMAL_FIT = 50000
+    ACCURATE_FIT = 100000
+    EXTREME_FIT = 10000000
 
-    def __init__(self, structure_a, structure_b, tolerance_cell_misfit = 0.1, tolerance_atomic_misfit = 1.0, supercells_allowed = True, anonymized = False):
+    def __init__(self, structure_a, structure_b, tolerance_cell_misfit = 0.1, tolerance_atomic_misfit = 1.0, supercells_allowed = True, anonymized = False, fitting_accuracy = FAST_FIT):
         """
         Fits two structures.
         All fitting parameters have been set with defaults that should work in most cases.
@@ -47,21 +54,35 @@ class StructureFitter(object):
         print fitter
         
         Args:
-            structure_a : First structure
-            structure_b : Second structure to try to match with first structure
-            tolerance_cell_misfit : Tolerance for cell misfit. Default = 0.1
-            tolerance_atomic_misfit : Tolerance for atomic misfit. Default = 1.0.
-            supercells_allowed : Whether supercell structures are allowed.  Default = True.
-            anonymized : Whether to attempt matching of different species.  Setting this to true
-                         will allow any two structures with the same framework, but different species
-                         to match to each other. Default = False.
-            el_mapping : Element mapping for anonymized fits.
+            structure_a : 
+                First structure
+            structure_b : 
+                Second structure to try to match with first structure
+            tolerance_cell_misfit : 
+                Tolerance for cell misfit. Default = 0.1
+            tolerance_atomic_misfit : 
+                Tolerance for atomic misfit. Default = 1.0.
+            supercells_allowed : 
+                Whether supercell structures are allowed.  Default = True.
+            anonymized : 
+                Whether to attempt matching of different species.  Setting this to true
+                will allow any two structures with the same framework, but different species
+                to match to each other. Default = False.
+            fitting_accuracy : 
+                An integer setting for the fitting accuracy.  Corresponds to the max number
+                of candidate rotations considered.  Use the static variables, 
+                    StructureFitter.FAST_FIT
+                    StructureFitter.NORMAL_FIT
+                    StructureFitter.ACCURATE_FIT
+                to set the tradeoff between accuracy and speed.  The default, FAST_FIT, should
+                work reasonably well in most instances.
         """
-    
+        
         self._tolerance_cell_misfit = tolerance_cell_misfit
         self._tolerance_atomic_misfit = tolerance_atomic_misfit
         self._supercells_allowed = supercells_allowed
         self._anonymized = anonymized
+        self._max_rotations = fitting_accuracy
         #Sort structures first so that they have the same arrangement of species
         self._structure_a = structure_a.get_sorted_structure()
         self._structure_b = structure_b.get_sorted_structure()
@@ -80,7 +101,7 @@ class StructureFitter(object):
                     # Create mapping of the specie/elements in structure B to that of A.
                     # Then create a modified structure with those elements and try to fit it.
                     el_mapping = dict(zip(comp_b.elements, p))
-                    logging.info("Using specie mapping " + str(el_mapping))
+                    logger.debug("Using specie mapping " + str(el_mapping))
                     mod = StructureEditor(self._structure_b)
                     mod.replace_species(el_mapping)
                     self.fit(self._structure_a, mod.modified_structure)
@@ -89,160 +110,84 @@ class StructureFitter(object):
                         self.el_mapping = {el_a:el_b for el_b, el_a in el_mapping.items()}
                         break                
             else:
-                logging.info("No. of elements in structures are unequal.  Cannot be fitted!")          
+                logger.debug("No. of elements in structures are unequal.  Cannot be fitted!")          
 
+    def identity_fit(self, a, b):
+        pass
+        
     def fit(self, a, b): 
         """
         Compares two structures and give the possible affine mapping that
         transforms one into another.
         """
         biggest_dist = 0
-        logging.info("Structure a")
-        logging.info(str(a))
-        logging.info("Structure b")
-        logging.info(str(b))
+        logger.debug("Structure a")
+        logger.debug(str(a))
+        logger.debug("Structure b")
+        logger.debug(str(b))
         
         # Check composition first.  If compositions are not the same, do not need to fit further.
         if a.composition.reduced_formula != b.composition.reduced_formula or ((a.num_sites != b.num_sites) and not self._supercells_allowed):
-            logging.info('Compositions do not match')
+            logger.debug('Compositions do not match')
             return None
         
-        logging.info('Compositions match')
+        logger.debug('Compositions match')
 
         # Fitting is done by matching sites in one structure (to_fit)
         # to the other (fixed).  
         # We set the structure with fewer sites as fixed, 
         # and scale the structures to the same density
-        (fixed, to_fit)  = self._scale_structures(a,b)
-        
+        (fixed, to_fit) = self._scale_structures(a,b)
         # Defines the atom misfit tolerance
         tol_atoms = self._tolerance_atomic_misfit * ( 3 * 0.7405 * fixed.volume / (4 * math.pi * fixed.num_sites)) ** (1 / 3)
-        logging.info("Atomic misfit tolerance = %.4f" % (tol_atoms) )
+        logger.debug("Atomic misfit tolerance = %.4f" % (tol_atoms) )
+        tol_atoms_plus = 1.1 * tol_atoms
         
-        max_sites = 1e100
+        max_sites = float('inf')
         # determine which type of sites to use for the mapping
         for sp in to_fit.species_and_occu:
             sp_sites = [site for site in to_fit if site.species_and_occu == sp]
             if len(sp_sites) < max_sites:
                 fit_sites = sp_sites
-                max_sites = len(sp_sites) 
-       
+                max_sites = len(sp_sites)
+        
         # Set the arbitrary origin
         origin = fit_sites[0]
-        logging.info("Origin = " + str(origin) )
-        
-        #Get candidate rotations
-        cand_rot = self._get_candidate_rotations(origin, fixed, to_fit)
-        
-        logging.info(" FOUND " + str(len(cand_rot)) + " candidate rotations ")
-        if len(cand_rot) == 0:
-            logging.info("No candidate rotations found, returning null. ")
-            return None
+        logger.debug("Origin = " + str(origin) )
         
         # now that candidate rotations have been found, shift origin of to_fit
         # because the symmetry operations will be applied from this origin
         oshift = SymmOp.from_rotation_matrix_and_translation_vector(np.eye(3), -origin.coords)
-        to_fit = apply_operation(to_fit, oshift)
-        # sort the operations, the first ones are the ones with small shear
-        # this assures us that we find the smallest cell misfit fits
-        sorted_cand_rot = sorted(cand_rot.keys(), key=lambda r: cand_rot[r])
-
-        tol_atoms_plus = 1.1 * tol_atoms
-        found_map = False
-        mapping_op = None
+        shifted_to_fit = apply_operation(to_fit, oshift)
         
-        for rot in sorted_cand_rot:
-            for site in fixed:
-                logging.info("Trying candidate rotation : \n" + str(rot))
-                if site.species_and_occu == origin.species_and_occu:
-                    shift = site.coords
-                    op = SymmOp.from_rotation_matrix_and_translation_vector(rot.rotation_matrix, shift)
-                    nstruct = apply_operation(to_fit, op)
-                    correspondance = OrderedDict()
-                    all_match = True
-                    biggest_dist = 0
-                    # check to see if transformed struct matches fixed structure
-                    for trans in nstruct:
-                        cands = fixed.get_sites_in_sphere(trans.coords, tol_atoms_plus)
-                        if len(cands) == 0:
-                            logging.info("No candidates found1")
-                            all_match = False
-                            break
-                        cands = sorted(cands, key = lambda a: a[1])
-                        (closest, closest_dist) = cands[0]
-                        if closest_dist > tol_atoms or closest.species_and_occu != trans.species_and_occu:
-                            logging.info("Closest dist too large! closest dist = " + str(closest_dist))
-                            all_match = False
-                            break
-                        correspondance[trans] = closest
-                        if closest_dist > biggest_dist:
-                            biggest_dist = closest_dist
-                    
-                    if not all_match:
-                        continue
-                    
-                    if not are_sites_unique(correspondance.values(), False):
-                        all_match = False
-                    else:
-                        for k,v in correspondance.items():
-                            logging.info(str(k) + " fits on " + str(v))
-                        
-                    # now check to see if the converse is true -- do all of the
-                    # sites of fixed match up with a site in toFit
-                    # this used to not be here. This fixes a bug.
-                    logging.info("Checking inverse mapping")
-                    inv_correspondance = OrderedDict()
-    
-                    # it used to be fixed.getNumSites() != nStruct.getNumSites()
-                    # so only when the number of sites are different but it's
-                    # actually better to allways check the reverse. This
-                    # elimininates weird situations where two atoms fit to one (reduced in the
-                    # unit cell)
-                    for fixed_site in fixed:
-                        cands = nstruct.get_sites_in_sphere(fixed_site.coords, tol_atoms_plus)
-                        if len(cands) == 0: 
-                            logging.info("Rejected because inverse mapping does not fit - Step 1")
-                            all_match = False
-                            break
-                        
-                        cands = sorted(cands, key = lambda a: a[1])
-                        (closest, closest_dist) = cands[0]
-                        
-                        if closest_dist > tol_atoms or closest.species_and_occu != fixed_site.species_and_occu:
-                            all_match = False
-                            logging.info("Rejected because inverse mapping does not fit - Step 2")
-                            break
-                        inv_correspondance[fixed_site] = closest
-                    
-                    if all_match:
-                        if not are_sites_unique(inv_correspondance.values(), False):
-                            all_match = False
-                            logging.info("Rejected because two atoms fit to the same site for the inverse")
-                    
-                    if all_match:
-                        self.inv_correspondance = inv_correspondance
-                        logging.info("Correspondance for the inverse")
-                        for k,v in inv_correspondance.items():
-                            logging.info(str(k) + " fits on " + str(v))
-                            
-                    # The smallest correspondance array shouldn't have any equivalent sites
-                    if fixed.num_sites != to_fit.num_sites:
-                        logging.info("Testing sites unique")
-                        if not are_sites_unique(correspondance.values()): 
-                            all_match = False
-                            logging.info("Rejected because the smallest correspondance array has equivallent sites")
-                            break
-        
-                    if all_match:
-                        found_map = True
-                        mapping_op = op
-                        self.correspondance = correspondance
-                        break
+        # This is cheating, but let's try the identity rotation first.  In many situations,
+        # E.g., when a structure has been topotatically delithiated or substituted, you actually
+        # can get a very fast answer without having to try all rotations.
+        identity_rot = SymmOp.from_rotation_matrix_and_translation_vector(np.eye(3), np.array([0,0,0]))
+        (found_map, mapping_op) = self._test_rot(identity_rot, origin, fixed, shifted_to_fit, tol_atoms, tol_atoms_plus)
+        if not found_map: #If identity matching does not work, we have to search and try all rotations.
+            logger.debug("Identity matching failed. Finding candidate rotations.")
+            #Get candidate rotations
+            cand_rot = self._get_candidate_rotations(origin, fixed, to_fit)
             
-            if found_map:
-                break
+            logger.debug(" FOUND {} candidate rotations ".format(len(cand_rot)))
+            if len(cand_rot) == 0:
+                logger.debug("No candidate rotations found, returning null. ")
+                return None
+            
+            # sort the operations, the first ones are the ones with small shear
+            # this assures us that we find the smallest cell misfit fits
+            sorted_cand_rot = sorted(cand_rot.keys(), key=lambda r: cand_rot[r])
+            
+            for rot in sorted_cand_rot:
+                (found_map, mapping_op) = self._test_rot(rot, origin, fixed, shifted_to_fit, tol_atoms, tol_atoms_plus)
+                
+                if found_map:
+                    break
+        else:
+            logger.debug("Identity matching found.")
         
-        logging.info("Done testing all candidate rotations")
+        logger.debug("Done testing all candidate rotations")
         
         self._atomic_misfit = biggest_dist / ((3 * 0.7405 * a.volume / (4 * math.pi * a.num_sites)) ** (1/3))
         if mapping_op != None:
@@ -258,7 +203,98 @@ class StructureFitter(object):
             self._mapping_op = composite_op if self.fixed_is_a else composite_op.inverse
             #self._mapping_op = mapping_op
             self._cell_misfit = shear_invariant(p)
-        
+
+    def _test_rot(self, rot, origin, fixed, to_fit, tol_atoms, tol_atoms_plus):
+        found_map = False
+        mapping_op = None
+        for site in fixed:
+            logger.debug("Trying candidate rotation : \n" + str(rot))
+            if site.species_and_occu == origin.species_and_occu:
+                shift = site.coords
+                op = SymmOp.from_rotation_matrix_and_translation_vector(rot.rotation_matrix, shift)
+                nstruct = apply_operation(to_fit, op)
+                correspondance = OrderedDict()
+                all_match = True
+                biggest_dist = 0
+                # check to see if transformed struct matches fixed structure
+                for trans in nstruct:
+                    cands = fixed.get_sites_in_sphere(trans.coords, tol_atoms_plus)
+                    if len(cands) == 0:
+                        logger.debug("No candidates found1")
+                        all_match = False
+                        break
+                    cands = sorted(cands, key = lambda a: a[1])
+                    (closest, closest_dist) = cands[0]
+                    if closest_dist > tol_atoms or closest.species_and_occu != trans.species_and_occu:
+                        logger.debug("Closest dist too large! closest dist = {}".format(closest_dist))
+                        all_match = False
+                        break
+                    correspondance[trans] = closest
+                    if closest_dist > biggest_dist:
+                        biggest_dist = closest_dist
+                
+                if not all_match:
+                    continue
+                
+                if not are_sites_unique(correspondance.values(), False):
+                    all_match = False
+                else:
+                    for k,v in correspondance.items():
+                        logger.debug(str(k) + " fits on " + str(v))
+                    
+                # now check to see if the converse is true -- do all of the
+                # sites of fixed match up with a site in toFit
+                # this used to not be here. This fixes a bug.
+                logger.debug("Checking inverse mapping")
+                inv_correspondance = OrderedDict()
+
+                # it used to be fixed.getNumSites() != nStruct.getNumSites()
+                # so only when the number of sites are different but it's
+                # actually better to allways check the reverse. This
+                # elimininates weird situations where two atoms fit to one (reduced in the
+                # unit cell)
+                for fixed_site in fixed:
+                    cands = nstruct.get_sites_in_sphere(fixed_site.coords, tol_atoms_plus)
+                    if len(cands) == 0: 
+                        logger.debug("Rejected because inverse mapping does not fit - Step 1")
+                        all_match = False
+                        break
+                    
+                    cands = sorted(cands, key = lambda a: a[1])
+                    (closest, closest_dist) = cands[0]
+                    
+                    if closest_dist > tol_atoms or closest.species_and_occu != fixed_site.species_and_occu:
+                        all_match = False
+                        logger.debug("Rejected because inverse mapping does not fit - Step 2")
+                        break
+                    inv_correspondance[fixed_site] = closest
+                
+                if all_match:
+                    if not are_sites_unique(inv_correspondance.values(), False):
+                        all_match = False
+                        logger.debug("Rejected because two atoms fit to the same site for the inverse")
+                
+                if all_match:
+                    self.inv_correspondance = inv_correspondance
+                    logger.debug("Correspondance for the inverse")
+                    for k,v in inv_correspondance.items():
+                        logger.debug("{} fits on {}".format(k,v))
+                        
+                # The smallest correspondance array shouldn't have any equivalent sites
+                if fixed.num_sites != to_fit.num_sites:
+                    logger.debug("Testing sites unique")
+                    if not are_sites_unique(correspondance.values()): 
+                        all_match = False
+                        logger.debug("Rejected because the smallest correspondance array has equivallent sites")
+                        break
+    
+                if all_match:
+                    found_map = True
+                    mapping_op = op
+                    self.correspondance = correspondance
+                    break
+                
+        return (found_map, mapping_op)
 
     def __str__(self):
         
@@ -284,7 +320,8 @@ class StructureFitter(object):
     def _scale_structures(self, a, b):
         # which structure do we want to fit to the other ?
         # assume that structure b has less sites and switch if needed
-        self.fixed_is_a = a.num_sites > b.num_sites
+               
+        self.fixed_is_a = a.num_sites  > b.num_sites 
         (fixed, to_fit) = (a, b) if self.fixed_is_a else (b, a)
             
         # scale the structures to the same density
@@ -306,30 +343,48 @@ class StructureFitter(object):
         for i in range(3):
             dr = lengths[i] * math.sqrt(tol_shear / 2)
             shell = to_fit.get_neighbors_in_shell(origin.coords, lengths[i], dr)
-            logging.info("shell " + str(i) + " radius=[" + str(lengths[i]) + "] dr=[" + str(dr) + "]")
-            shells.append([site for (site, dist) in shell if site.species_and_occu == origin.species_and_occu])
-            logging.info("No. in shell = " + str(len(shells[-1])))
+            logger.debug("shell {} radius={} dr={}".format(i, lengths[i], dr))
+            shell = filter(lambda x: x[0].species_and_occu == origin.species_and_occu, shell)
+            shell = sorted(shell, key = lambda x: x[1])
+            shells.append([site for (site, dist) in shell])
+            logger.debug("No. in shell = {}".format(len(shells[-1])))
         # now generate candidate rotations
         cand_rot = {} #Dict of SymmOp : float
-        
-        for pool in itertools.product(*shells):
-            if all([nn.species_and_occu == origin.species_and_occu for nn in pool]):
-                # now, can a unitary transformation bring the cell vectors together
-                cell_v = np.array([nn.coords - origin.coords for nn in pool]).transpose()
-                det = np.linalg.det(cell_v)
-                if abs(det) < 0.001 or abs(abs(det) - fixed.volume) > 0.01:
-                    continue
-                rot = np.dot(fixed_basis, np.linalg.inv(cell_v))
-                r = SymmOp.from_rotation_matrix_and_translation_vector(rot, np.array([0,0,0]))
+        a = len(shells[0])
+        b = len(shells[1])
+        c = len(shells[2])
+        total_rots = a * b * c
+        if total_rots < self._max_rotations:
+            logger.debug("Total rots = {}. Using all rotations.".format(total_rots))
+            test_rotations = itertools.product(*shells)
+        else:
+            logger.warning("Total rots = {m} exceed max_rotations = {n}. Using {n} randomly selected rotations.".format(m = total_rots, n = self._max_rotations))
+            def random_rot():
+                considered_rots = []
+                while len(considered_rots) < self._max_rotations:
+                    (x, y, z) = [random.randint(0,i-1) for i in [a,b,c]]
+                    if (x,y,z) not in considered_rots:
+                        considered_rots.append((x,y,z))
+                        yield (shells[0][x], shells[1][y], shells[2][z])
+            test_rotations = random_rot()
+            
+        for pool in test_rotations:
+            # now, can a unitary transformation bring the cell vectors together
+            cell_v = np.array([nn.coords - origin.coords for nn in pool]).transpose()
+            det = np.linalg.det(cell_v)
+            if abs(det) < 0.001 or abs(abs(det) - fixed.volume) > 0.01:
+                continue
+            rot = np.dot(fixed_basis, np.linalg.inv(cell_v))
+            r = SymmOp.from_rotation_matrix_and_translation_vector(rot, np.array([0,0,0]))
+
+            if r not in cand_rot:
+                transf = r.rotation_matrix
+                transf = np.dot(transf.transpose(), transf)
+                transf = np.eye(3) if almost_identity(transf) else transf
+                pbis = sqrt_matrix(transf)
+                if shear_invariant(pbis) < tol_shear:
+                    cand_rot[r] = shear_invariant(pbis)
     
-                if r not in cand_rot:
-                    transf = r.rotation_matrix
-                    transf = np.dot(transf.transpose(), transf)
-                    transf = np.eye(3) if almost_identity(transf) else transf
-                    pbis = sqrt_matrix(transf)
-                    if shear_invariant(pbis) < tol_shear:
-                        cand_rot[r] = shear_invariant(pbis)
-        
         return cand_rot
 
     @property
