@@ -913,7 +913,6 @@ class Composition (collections.Mapping, collections.Hashable):
     """
     amount_tolerance = 1e-8
     
-    
     """
     Special formula handling for peroxides and certain elements. This is so that 
     formula output does not write LiO instead of Li2O2 for example.
@@ -1024,7 +1023,7 @@ class Composition (collections.Mapping, collections.Hashable):
     @property
     def formula(self):
         '''
-        Returns a formula string, e.g. Li4 Fe4 P4 O16.
+        Returns a formula string, with elements sorted by electronegativity e.g. Li4 Fe4 P4 O16.
         '''
         elements = self._elmap.keys()
         elements = sorted(elements,key=lambda el: el.X)
@@ -1034,6 +1033,19 @@ class Composition (collections.Mapping, collections.Hashable):
                 formula.append(el.symbol+formula_double_format(self[el], False))
         return ' '.join(formula)
     
+    @property
+    def alphabetical_formula(self):
+        '''
+        Returns a formula string, with elements sorted by alphabetically e.g. Fe4 Li4 O16 P4.
+        '''
+        elements = self._elmap.keys()
+        elements = sorted(elements,key = lambda el: el.symbol)
+        formula = []
+        for el in elements:
+            if self[el] != 0:
+                formula.append(el.symbol+formula_double_format(self[el], False))
+        return ' '.join(formula)
+    
     def get_reduced_formula_and_factor(self):
         '''
         Returns a pretty normalized formula and a multiplicative factor, i.e., Li4Fe4P4O16 returns (LiFePO4, 4).
