@@ -296,9 +296,16 @@ class StructureVis(object):
             for (vec1, vec2, vec3) in itertools.permutations(matrix, 3):
                 self.add_line(vec1 + vec2, vec1 + vec2 + vec3)
         
+        
+        
         if self.show_bonds or self.show_polyhedron:
             elements = sorted(s.composition.elements, key = lambda a: a.X)
             anion = elements[-1]
+            def contains_anion(site):
+                for sp, occu in site.species_and_occu.items():
+                    if sp.symbol == anion.symbol:
+                        return True
+                return False
             anion_radius = anion.average_ionic_radius
             for site in s:
                 exclude = False
@@ -316,9 +323,10 @@ class StructureVis(object):
                     nn = structure.get_neighbors(site, max_radius)
                     nn_sites = []
                     for nnsite, dist in nn:
-                        nn_sites.append(nnsite)
-                        if not in_coord_list(inc_coords, nnsite.coords):
-                            self.add_site(nnsite)
+                        if contains_anion(nnsite):
+                            nn_sites.append(nnsite)
+                            if not in_coord_list(inc_coords, nnsite.coords):
+                                self.add_site(nnsite)
                     if self.show_bonds:
                         self.add_bonds(nn_sites, site.x, site.y, site.z)
                     if self.show_polyhedron:
@@ -393,7 +401,7 @@ class StructureVis(object):
                 color = [i/255 for i in self.el_color_mapping[specie.symbol]]
             actor.GetProperty().SetColor(color)
             self.ren.AddActor(actor)
-
+        
         for specie, occu in site.species_and_occu.items():
             add_partial_sphere(start_angle, start_angle + 360 * occu, specie)
             start_angle = start_angle + 360 * occu
@@ -414,7 +422,7 @@ class StructureVis(object):
         self.ren.AddActor(follower)
         follower.SetCamera(self.ren.GetActiveCamera())
 
-    def add_line(self, start, end, color = (0.5,0.5,0.5)):
+    def add_line(self, start, end, color = (0.5,0.5,0.5), width = 1):
         source = vtk.vtkLineSource()
         source.SetPoint1(start)
         source.SetPoint2(end)
@@ -432,8 +440,10 @@ class StructureVis(object):
         actor = vtk.vtkActor()
         actor.SetMapper(mapper)
         actor.GetProperty().SetColor(color)
+        actor.GetProperty().SetLineWidth(width)
         self.ren.AddActor(actor)
 
+
     def add_polyhedron(self, neighbours, center, color):
         points = vtk.vtkPoints()
         conv = vtk.vtkConvexPointSet()
@@ -538,7 +548,8 @@ class StructureVis(object):
         picker.AddObserver("EndPickEvent", annotate_pick)
         self.picker = picker
         self.iren.SetPicker(picker)
-
+        
+                
 def make_movie(structures, output_filename = "movie.mp4", zoom = 1.0, fps = 20, bitrate = 10000, quality = 5):
     """
     Generate a movie from a sequence of structures using vtk and ffmpeg.
